/*
 * Connect 4 class
 */

package edu.umn.d.cs5541.program2;

import java.util.LinkedList;

/**
 * Class for a simple Connect 4 game
 * @author rmaclin
 */
public class ConnectFourBoard {
    // Constants defining this version of Connect 4
    private static final int EMPTY = 0;
    private static final int BLACK = 1;
    private static final int RED = 2;
    private static final int MAX_COLUMN = 7;
    private static final int MAX_ROW = 6;
    private static final int TO_WIN = 4;

    // Store contents of the board
    private int[][] boardLocation;
    private int[] topRow;

    /**
     * Routine to initialize game board as empty
     */
    public void initialize() {
        boardLocation = new int[MAX_ROW][MAX_COLUMN];
        topRow = new int[MAX_COLUMN];
    }

    public ConnectFourBoard() { }

    /**
     * Returns a linked list of the possible moves (the numbers of the columns
     * that can accept a piece)
     * @return
     */
    public LinkedList legalMoves() {
        LinkedList moveList = new LinkedList();
        for (int c = 0; c < MAX_COLUMN; c++)
            if (topRow[c] < MAX_ROW)
                moveList.add(c);
        return moveList;
    }

    /**
     * Generic routine to make a move by adding a piece of color color at
     * column col
     * @param col
     * @param color
     * @throws Throwable
     */
    private void makeMove(int col, int color) throws Throwable {
        if (topRow[col] < MAX_ROW) {
            boardLocation[topRow[col]][col] = color;
            topRow[col] = topRow[col] + 1;
        }
        else
            throw new Throwable("Cannot add piece to column " + col);
    }

    /**
     * Routine to add black piece to column col
     * @param col
     * @throws Throwable
     */
    public void makeBlackMove(int col) throws Throwable {
        makeMove(col,BLACK);
    }

    /**
     * Routine to add red piece to column col
     * @param col
     * @throws Throwable
     */
    public void makeRedMove(int col) throws Throwable {
        makeMove(col,RED);
    }

    /**
     * Generic routine to check if player with color pieces has won game
     * @param color
     * @return
     */
    private boolean winForColor(int color) {
        for (int c = 0; c <= (MAX_COLUMN - TO_WIN); c++)
            for (int r = 0; r < MAX_ROW; r++) {
                boolean found = true;
                for (int i = 0; found && (i < TO_WIN); i++)
                    if (boardLocation[r][c+i] != color) found = false;
                if (found) return true;
            }
        for (int r = 0; r <= (MAX_ROW - TO_WIN); r++)
            for (int c = 0; c < MAX_COLUMN; c++) {
                boolean found = true;
                for (int i = 0; found && (i < TO_WIN); i++)
                    if (boardLocation[r+i][c] != color) found = false;
                if (found) return true;
            }
       for (int c = 0; c <= (MAX_COLUMN - TO_WIN); c++)
            for (int r = 0; r < (MAX_ROW - TO_WIN); r++) {
                boolean found = true;
                for (int i = 0; found && (i < TO_WIN); i++)
                    if (boardLocation[r+i][c+i] != color) found = false;
                if (found) return true;
            }
       for (int c = (MAX_COLUMN - TO_WIN); c < MAX_COLUMN; c++)
            for (int r = 0; r < (MAX_ROW - TO_WIN); r++) {
                boolean found = true;
                for (int i = 0; found && (i < TO_WIN); i++)
                    if (boardLocation[r+i][c-i] != color) found = false;
                if (found) return true;
            }
        return false;
    }

    /**
     * Check if black has won
     * @return
     */
    public boolean winForBlack() {
        return winForColor(BLACK);
    }

    /**
     * Check if red has won
     * @return
     */
    public boolean winForRed() {
        return winForColor(RED);
    }

    /**
     * Show a simple representation of the current board
     */
    public void showBoard() {
        for (int r = MAX_ROW - 1; r >= 0; r--) {
            System.out.print("|");
            for (int c = 0; c < MAX_COLUMN; c++) {
                switch (boardLocation[r][c]) {
                    case EMPTY: System.out.print(" |"); break;
                    case BLACK: System.out.print("B|"); break;
                    case RED: System.out.print("r|"); break;
                    default: System.out.println("Problem value is " + boardLocation[r][c]); break;
                }
            }
            System.out.println();
        }
        for (int c = 0; c < MAX_COLUMN; c++)
            System.out.print(" " + c);
        System.out.println();
    }
}
