/*
 * Main class for the Connect 4 Game Class
 */
package edu.umn.d.cs5541.program2;

import edu.umn.d.cs5541.program2.ConnectFourBoard;
import java.io.BufferedReader;
import java.io.EOFException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.LinkedList;

/**
 * Class provides static methods for running a connect 4 game
 * @author rmaclin
 */
public class Main {
    // Input stream variable
    private static BufferedReader bufferedReader = null;
    
    // Game Board used for game
    private static ConnectFourBoard gameBoard = null;
    
    // Variable to hold user input
    private static int ch = 0;

    /**
     * Routine to read next meaningful character
     * @return
     * @throws IOException
     */
    private static int getNextChar() throws IOException {
        if (bufferedReader == null) {
            //  open up standard input
            bufferedReader = new BufferedReader(new InputStreamReader(System.in));
        }

        do {
            try {
                ch = bufferedReader.read();
            } catch (EOFException e) {
                return -1;
            } catch (IOException e) {
                throw e;
            }
        } while (Character.isWhitespace(ch));

        bufferedReader.readLine();

        return ch;
    }

    /**
     * Routine plays one game
     * @throws Throwable
     */
    private static void playGame() throws Throwable {
        LinkedList legalMoves = gameBoard.legalMoves();
        gameBoard.showBoard();
        int move = 0;
        while (!(legalMoves.isEmpty())) {
            // Make red move
            do {
                System.out.print("Red move? " + legalMoves + " ");
                move = getNextChar();
                if ((move == 'q') || (move == 'Q')) return;
                move = move - '0';
            } while (!(legalMoves.contains(move)));
            gameBoard.makeRedMove(move);
            gameBoard.showBoard();
            if (gameBoard.winForRed()) {
                System.out.println("Red Wins!");
                return;
            }
            legalMoves = gameBoard.legalMoves();
            
            // Make black move
            do {
                System.out.print("Black move? " + legalMoves + "] ");                
                move = getNextChar();
                if ((move == 'q') || (move == 'Q')) return;
                move = move - '0';
            } while (!(legalMoves.contains(move)));
            gameBoard.makeBlackMove(move);
            gameBoard.showBoard();
            if (gameBoard.winForBlack()) {
                System.out.println("Black Wins!");
                return;
            }
            legalMoves = gameBoard.legalMoves();
        }
        System.out.println("Draw!");
    }

    /**
     * Main routine to play a series of Connect 4 games
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        gameBoard = new ConnectFourBoard();
        do {
            gameBoard.initialize();

            try {
                playGame();
                System.out.print("Play again? [y/n] ");
                ch = getNextChar();
            } catch (Throwable e) {
                System.err.println(e.getMessage());
                return;
            }
        } while ((ch == 'Y') || (ch == 'y'));
    }
}
